/* 
Wicket - dodgy Java IRC bot
Copyright Mat Grove, 2004, http://www.matthewgrove.co.uk/

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation.

*/
package uk.ac.port.dsg.wicket;

import java.util.*;
import java.io.*;
import java.lang.reflect.*;
import java.net.*;
import uk.ac.port.dsg.wicket.plugins.BrainInterface;

public class PluginManager {

	public static ArrayList loadPlugins(Brain bot) {
		//System.out.println("Discovering plugins");
		ArrayList loadedObjects = new ArrayList();
		Vector modules = discoverPlugins("WicketPlugin");
		for (int i = 0; i < modules.size(); i++) {
			String plugin = (String) modules.get(i);

			Object loadedObject = createDynamicObject(plugin, bot);
			if (loadedObject != null) {
				loadedObjects.add(loadedObject);
			} else {
				System.out.println("Failed to load plugin: " + plugin);
			}

		}
		if (loadedObjects.isEmpty()) {
			// when ever we ask for a plugin if we return nothing teh program will bust
			// there should always be a plugin to saticfy the query
			// so we stop
			System.out.println("Can't load at least one plugin");
			return null;
		} else {
			// sort the list before we return it

			ArrayList sortedList = new ArrayList();

			while (loadedObjects.size() > 0) {
				int smallestObject = 0;
				int smallestVal = 1000;
				for (int outer = 0; outer < loadedObjects.size(); outer++) {
					int compare = ((BrainInterface) loadedObjects.get(outer)).getImportance();
					if (compare < smallestVal) {
						smallestVal = compare;
						smallestObject = outer;
					}
				}
				sortedList.add(loadedObjects.remove(smallestObject));
			}
			return sortedList;
		}
	}

	private static Object createDynamicObject(String className, Brain bot) {
		try {

			//System.out.println("Trying to load: " + className);

			// get reference to the class

			File file = new File(".");
			ClassLoader loader = new URLClassLoader(new URL[] { file.toURL()});
			Class classRef = loader.loadClass(className);

			//Class classRef = Class.forName(className);

			Class[] classConst = new Class[1];
			classConst[0] = Class.forName("uk.ac.port.dsg.wicket.Brain");

			Object[] classObject = new Object[1];
			classObject[0] = bot;

			Constructor constr = classRef.getConstructor(classConst);
			Object newObject = constr.newInstance(classObject);

			if (newObject != null) {
				System.out.println("Dynamicaly loaded: " + className);
			}

			return newObject;

		} catch (java.lang.ClassNotFoundException e) {
			System.out.println("Couldn't find class: " + className);
			return null;
		} catch (java.lang.Exception e) {
			System.out.println("Couldn't dynamically load class: " + className);
			System.out.println(e.toString());
			e.printStackTrace();
			return null;
		}

	}

	private static Vector discoverPlugins(String classOfPlugin) {
		String rootPluginPath = "uk/ac/port/dsg/wicket/plugins/";
		String classPathStub = "uk.ac.port.dsg.wicket.plugins.";

		Vector foundModules = new Vector();

		File dir = new File(rootPluginPath);
		// This filter only returns directories
		FileFilter fileFilter = new FileFilter() {
			public boolean accept(File file) {
				return file.isDirectory();
			}
		};
		File[] dirs = dir.listFiles(fileFilter);

		for (int i = 0; i < dirs.length; i++) {
			// Get filename of file or directory
			String dirName = dirs[i].getName();
			//System.out.println("Found dir: " + dirName);
			//if (!filterCmd(filter, dirName)) {

			String pathToClass = rootPluginPath + dirName + File.separator + classOfPlugin;
			String pathToClassFileName = pathToClass + ".class";
			//System.out.println("Module to look for is:" + pathToClassFileName);
			File moduleFile = new File(pathToClassFileName);
			if (moduleFile.exists()) {
				//System.out.println("Could load: " + pathToClassFileName);
				pathToClass = pathToClass.replaceAll(rootPluginPath, "");
				foundModules.add(classPathStub + pathToClass.replaceAll(File.separator, "."));
			}
			//}
		}

		return foundModules;

	}

}
